package ratRace.viewSwing;
import java.awt.BorderLayout;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;

import ratRace.controller.Controller;
import ratRace.model.Model;

/** This is the "main" class and also the class of the main window.
 * In a sense this class also serves as a higher-level controller, 
 * controlling start-up and transitions between levels, each level
 * being one model (maze and rat), its view, and controller.
 * This class functions as an "executive", being responsible for
 * creating the model, view, and controller, and for hooking them together.
 * The RatRace object observes the model in order to keep its buttons'
 * enabled/disabled status in sync with the model.
 * 
 * @See controller.Controller view.View model.Model
 */
public class RatRace extends JFrame implements Observer {
	private Model model ;
	private SwingView view ;
	private Controller controller ;
	private JButton goButton = new JButton("Go") ;
	private JButton pauseButton = new JButton("Pause") ;
	private JButton restartButton = new JButton("Restart") ;
		
	void rebuildMVC() {
		if( view != null ) { view.terminate(); remove( view ) ; }
		model = new Model(50, 50) ;
		model.addObserver( this ) ;
		view = new SwingView( model ) ;
		controller = new Controller(model, view) ;
		add( view ) ;
		setVisible( true ) ;
		update(null, null) ; }
	
	public RatRace() {
		setDefaultCloseOperation(EXIT_ON_CLOSE) ;
		
		goButton.addActionListener(
			actionEvent -> controller.go() ) ;
		pauseButton.addActionListener(
			actionEvent -> controller.pause() ) ;
		restartButton.addActionListener( 
			actionEvent -> rebuildMVC() ) ;
		
		JToolBar toolBar = new JToolBar() ;
		toolBar.add( restartButton ) ;
		toolBar.add(goButton ) ;
		toolBar.add( pauseButton ) ;
		add( toolBar, BorderLayout.NORTH ) ;

		
		setSize( 500, 500 ) ;

		rebuildMVC() ;
	}

	@Override
	public void update(Observable o, Object arg) {
		goButton.setEnabled( model.getPaused() ) ;
		pauseButton.setEnabled( !model.getPaused() ) ;
		if( model.getPaused() ) {
			goButton.requestFocusInWindow() ; }
		else {
			pauseButton.requestFocusInWindow() ; } }
	
	public static void main(String[] args) {
		SwingUtilities.invokeLater( new Runnable() {
			@Override public void run() {
				new RatRace() ;
			}} ) ;
	}
}