package ratRace.viewSwing;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import ratRace.model.DrawingAdapter;

public class SwingDrawingAdapter implements DrawingAdapter {

	private final Graphics2D g2d;
	private final Transform xform ;
	private final double width;
	private final double height;

	SwingDrawingAdapter(  Graphics2D g2d, Transform xform, double width, double height ) {
		this.g2d = g2d ;
		this.xform = xform ;
		this.width = width ;
		this.height = height ;
	}
	
	@Override
	public void putMessage(String message, boolean gameOver ) {
		g2d.setColor( new Color(0, 0, 255, 100) ) ;
		if( gameOver ) {
			Font bigFont = new Font(Font.MONOSPACED, Font.PLAIN, 100 ) ;
			FontMetrics fm = g2d.getFontMetrics(bigFont) ;
			int wf = fm.stringWidth(message) ;
			int xf = (int)( width/2 - wf/2 ) ;
			int hf = fm.getAscent() ;
			int yf = (int)( height/2 + hf/2 ) ;
			g2d.setFont( bigFont ) ;
			g2d.drawString(message, xf, yf) ;
		} else {
			Font smallFont = new Font(Font.MONOSPACED, Font.PLAIN, 20 ) ;
			FontMetrics fm = g2d.getFontMetrics(smallFont) ;
			int wf = fm.stringWidth(message) ;
			int xf = (int)( width*0.9 ) - wf ;
			int hf = fm.getAscent() ;
			int yf = (int)( height*0.1 ) + hf ;
			g2d.setFont( smallFont ) ;
			g2d.drawString(message, xf, yf) ; }
	}

	@Override
	public void drawWall(double i0, double j0, double i1, double j1) {
		Point2D p0 = xform.worldToView(i0, j0) ;
		Point2D p1 = xform.worldToView(i1, j1) ;
		Line2D line = new Line2D.Double(p0, p1) ;
		g2d.draw( line ) ; 
	}

	@Override
	public void drawGoal(double i, double j) {
		Point2D p0 = xform.worldToView(i, j) ;
		Point2D p1 = xform.worldToView(i+1, j+1) ;
		double x = p0.getX(), y = p0.getY();
		double w = p1.getX()-x, h = p1.getY()-y ;
		Rectangle2D rect = new Rectangle2D.Double(x, y, w, h ) ;
		g2d.setColor( Color.BLUE ) ;
		g2d.fill( rect ); 
		g2d.setColor( Color.BLACK ) ;
		g2d.draw( rect ); 
	}

	@Override
	public void drawRat(double i, double j, double radius) {
		Point2D p0 = xform.worldToView(i-radius, j-radius) ;
		Point2D p1 = xform.worldToView(i+radius, j+radius) ;
		double x = p0.getX(), y = p0.getY();
		double w = p1.getX()-x, h = p1.getY()-y ;
		Ellipse2D circle = new Ellipse2D.Double(x, y, w, h ) ;
		g2d.setColor( Color.RED ) ;
		g2d.fill( circle ); 
		g2d.setColor( Color.BLACK ) ;
		g2d.draw( circle ); 
	}

}
